﻿
////////////////////////////////////////////////////////////////////////////////////////////////////// sb_lightWrap //////////////////////////////////////////////////////////////////////////////////////////////////////////////
// sb_lightwrap
// by Simon Björk
// Version 2.0
// February 2012
// bjork.simon@gmail.com

// Based on a technique described in Mark Christiansens book "After Effects CS4 Studio Techniques" with a few added features. 

// Select your foreground and background layers and run the script.

////////////////////////////////////////////////////////////////////////////////////////////////////// User variables start //////////////////////////////////////////////////////////////////////////////////////////////////////////////

// Edit the user variables to your preferences.

var UV1 = "50"; // Default blurriness.
var UV2 = "20"; // Default opacity.
var UV3 = "Add"; // Default blending mode. Choose between Add/Screen/Lighten.
var UV4 = true; // Link transform values. Set true/false.
var UV5 = true; // Use luma key. Set true/false.
var UV6 = true; // Add luma key controls in main composition. Set true/false.

////////////////////////////////////////////////////////////////////////////////////////////////////// User variables end //////////////////////////////////////////////////////////////////////////////////////////////////////////////

////////////////////////////////////////////////////////////////////////////////////////////////////// User interface start //////////////////////////////////////////////////////////////////////////////////////////////////////////////

// Build UI
var myPalette = buildUI(this);

    if(myPalette != null && myPalette instanceof Window) {
       myPalette.show();
    }

function buildUI (thisObject) {
    
    if(thisObject instanceof Panel) {
        var myPalette = thisObject;
        }else{
            var myPalette = new Window ("palette", "sb_lightWrap", undefined, {resizeable:true});
            }

    if (myPalette != null) {
    
    var res =
    "Group { \
    orientation: 'column', \
    alignment: ['fill', 'fill'], \
    alignChildren: ['left', 'top'], \
    minimumSize: [125, 170],\
        one: Group { \
            orientation: 'row', \
            blurrinessTxt: StaticText {text: 'Blurriness:'},\
            opacityTxt: StaticText {text: ' Opacity:'},\
            } \
        two: Group { \
            orientation: 'row', \
            blurriness: EditText {size: ['50', '25']}, \
            opacity: EditText { size: ['50', '25'] }, \
            } \
        three: Group { \
            orientation: 'row', \
            blending: DropDownList {text: 'Blending mode', size: ['100', '25']}, \
            } \
        four: Group { \
            orientation: 'column', \
            transform: Checkbox {text: 'Link transforms'},\
            } \
        five: Group { \
            orientation: 'column', \
            luma: Checkbox {text: 'Use luma key'},\
            } \
        six: Group { \
            orientation: 'row', \
            lightwrap: Button {text: 'Create light wrap', size: ['120', '25']}, \
            } \
    }";

      myPalette.grp = myPalette.add(res);
      myPalette.layout.layout(true);
      myPalette.layout.resize()
      myPalette.resizing = myPalette.onResize = function () {this.layout.resize()};      

var blendingArray = new Array ("Add", "Screen", "Lighten");

for(i=0; i<blendingArray.length; i++) {
    myPalette.grp.three.blending.add('item',blendingArray[i]);
    }

// Find the default blending mode.
for(var i = 0; i< myPalette.grp.three.blending.items.length; i++){
    if(myPalette.grp.three.blending.items[i].toString() == UV3){
        var defaultBlendingMode = i;
        break;
        }else{
            var defaultBlendingMode = 0;
            } // if(myPalette.grp.three.blending.items[i].toString() == UV3){
        } // for(var i = 0; i< myPalette.grp.three.blending.items.length; i++){

// Set defaullt values for UI.
myPalette.grp.three.blending.selection = myPalette.grp.three.blending.items[defaultBlendingMode];
myPalette.grp.four.transform.value = UV4;
myPalette.grp.five.luma.value = UV5;
myPalette.grp.two.blurriness.text = UV1;
myPalette.grp.two.opacity.text = UV2;

// Run lightwrap function.
myPalette.grp.six.lightwrap.onClick = function () {
    lightwrap (myPalette);
    }

}//if(myPalette != null) {

return myPalette;
    
} //buildUI

////////////////////////////////////////////////////////////////////////////////////////////////////// User interface end //////////////////////////////////////////////////////////////////////////////////////////////////////////////

////////////////////////////////////////////////////////////////////////////////////////////////////// Lightwrap function start //////////////////////////////////////////////////////////////////////////////////////////////////////////////

function lightwrap (palObject) {

var myComp = app.project.activeItem;    
var selLayers = myComp.selectedLayers;

//Check to see if more than one layer is selected.
    if(selLayers.length<2){
        alert("Please select your foreground and background layers.");
        return;
        }

//Check if there are any 3D layers selected.
for(var i = 0; i<selLayers.length; i++){
    if(selLayers[i].threeDLayer == true){
        alert("One of the selected layers is a 3D layer. The light wrap effect won't work as expected, as the Stencil and Siluette blending modes doesn't work well with 3D layers. Take a look in the light wrap composition and see if you can fix it manually. To precompose the 3D layer is a possible solution.");
        break;
    }
}

// Sort the array by layer index.
function sortLayersByIndex(a, b){
    return a.index - b.index;
    }
selLayers.sort(sortLayersByIndex); 

// Create a name for the lightwrap composition. Make sure it's unique if there are several lightwraps for the same layer.
var fgLayer = selLayers[0];
var fgLayerName = fgLayer.name.toString()
var nameLength = parseFloat(fgLayer.name.length) + 10;

var lwNameArray = new Array();
for(var i = 1; i<=app.project.numItems; i++){ // Put every composition that starts with fg_layername_lightwrap into an array.
    var compName = app.project.item(i).name.toString();
    if(app.project.item(i) instanceof CompItem){
        var firstChars = compName.substr(0, (nameLength));
        if(firstChars == fgLayerName + "_lightwrap"){
            lwNameArray[lwNameArray.length] = app.project.item(i);
            } // if(firstChars == fgLayerName + "_lightwrap" ||){
        } // if(app.project.item(i) instanceof CompItem){
    } // for(var i = 1; i<=app.project.numItems; i++){

// Name if it's the first light wrap.
    if(lwNameArray.length == 0){
        var lightwrapName = fgLayerName + "_lightwrap";
        }

// Name if it's the second light wrap.
    if(lwNameArray.length == 1 && lwNameArray[0].name == fgLayerName + "_lightwrap"){
        var lightwrapName = fgLayerName + "_lightwrap_02";
        }

// Name if it's the third or above light wrap.
    if(lwNameArray.length > 1){
        var numbersArray = new Array();
        for(var i = 0; i< lwNameArray.length; i++){ // // Puts the last numbers into an array.
            var lastChar = lwNameArray[i].name.slice(nameLength + 1);
            numbersArray[numbersArray.length] = lastChar;
            }
        numbersArray.sort(function(a,b){return a - b}) // Sort the array. 
        numbersArray.reverse();
        var nextNumber = parseFloat(numbersArray[0]) + 1;
            if(nextNumber < 10){
                var lightwrapName = fgLayerName + "_lightwrap_0" + nextNumber;
                }else{
                    var lightwrapName = fgLayerName + "_lightwrap_" + nextNumber;
                    }            
        } // if(lwNameArray.length > 1){

// Create undo command.
app.beginUndoGroup("sb_lightWrap");

//Create the light wrap composition.
var lwComp = app.project.items.addComp(lightwrapName, myComp.width, myComp.height, myComp.pixelAspect , myComp.duration, myComp.frameRate);

// Sort the array by layer index.
function sortLayersByIndex(a, b){
    return a.index - b.index;
    }
selLayers.sort(sortLayersByIndex); 

// Copy layers to new compositon, beginning from the bottom. Also set expression links for the transform propertys.
for (var i=selLayers.length-1; i>=0; i--) {
    var lwLayers = selLayers[i];
    lwLayers.copyToComp(lwComp);
    if(myPalette.grp.four.transform.value == true){ // If link transform is checked:
        lwComp.layer(1).scale.expression = "comp(\"" + myComp.name + "\").layer(\"" + selLayers[i].name + "\").transform.scale";
        lwComp.layer(1).position.expression = "comp(\"" + myComp.name + "\").layer(\"" + selLayers[i].name + "\").transform.position";
        lwComp.layer(1).rotation.expression = "comp(\"" + myComp.name + "\").layer(\"" + selLayers[i].name + "\").transform.rotation";
        lwComp.layer(1).opacity.expression = "comp(\"" + myComp.name + "\").layer(\"" + selLayers[i].name + "\").transform.opacity";
        } // if(myPalette.grp.one.transform.value == true){
    } // for (var i=selLayers.length-1; i>=0; i--) {

// Create the light wrap.
lwComp.layer(1).duplicate();
lwComp.layer(1).blendingMode = BlendingMode.STENCIL_ALPHA;
lwComp.layer(2).blendingMode = BlendingMode.SILHOUETE_ALPHA;

var blurLayer = lwComp.layers.addSolid([1,1,1], "blurriness", lwComp.width, lwComp.height, lwComp.pixelAspect, lwComp.duration);
blurLayer.adjustmentLayer = true; 
blurLayer.moveBefore(lwComp.layer(3));
blurLayer.property("Effects").addProperty("Fast Blur");

var removeAlpha = lwComp.layers.addSolid([1,1,1], "remove_alpha", lwComp.width, lwComp.height, lwComp.pixelAspect, lwComp.duration);
removeAlpha.adjustmentLayer = true; 
removeAlpha.property("Effects").addProperty("Solid Composite") ("Color").setValue([0,0,0]);

// Add the light wrap to the original composition.
var lwLayer = myComp.layers.add(lwComp);
var fgLayer = selLayers[0];
myComp.layer(1).moveBefore(fgLayer);

// Set blending mode.
    if(myPalette.grp.three.blending.selection.toString() == "Add"){
        lwLayer.blendingMode = BlendingMode.ADD;
        }else if(myPalette.grp.three.blending.selection.toString() == "Screen"){
            lwLayer.blendingMode = BlendingMode.SCREEN;
        }else if(myPalette.grp.three.blending.selection.toString() == "Lighten"){
            lwLayer.blendingMode = BlendingMode.LIGHTEN;
        } // if(myPalette.grp.three.blending.selection.toString() == "Add"){

// Add controls for blurriness and set a default values.
var blurControl = lwLayer.property("Effects").addProperty("Slider Control")
var blurControl_name = lwLayer.effect("Slider Control").name = "blurriness";
blurLayer.effect("Fast Blur") ("Blurriness").expression = "comp(\"" + myComp.name + "\").layer(\"" + lwComp.name + "\").effect(\"" + blurControl_name + "\")" + "(\"Slider\")"

var defaultBlur = parseFloat(myPalette.grp.two.blurriness.text);
lwLayer.effect("blurriness") ("Slider").setValue(defaultBlur);

// Set default value for opacity.
var defaultOpacity = parseFloat(myPalette.grp.two.opacity.text);
lwLayer.opacity.setValue(defaultOpacity);

// Add luma key layer.
    if(myPalette.grp.five.luma.value == true){
        var lumakeyLayer= lwComp.layers.addSolid([1,1,1], "luma_key", lwComp.width, lwComp.height, lwComp.pixelAspect, lwComp.duration);
        lumakeyLayer.adjustmentLayer  = true;
        lumakeyLayer.moveAfter(lwComp.layer(5));
        var alphaFromLuma = lumakeyLayer.property("Effects").addProperty("Shift Channels");
        alphaFromLuma("Take Alpha From").setValue(5);
        var lumaLevels = lumakeyLayer.property("Effects").addProperty("Levels (Individual Controls)");
        lumaLevels("Channel:").setValue(5);
        if(UV6 == true){
            // Add controls in main composition.
            var luma_adj = lwLayer.property("Effects").addProperty("Levels (Individual Controls)");
            luma_adj("Channel:").setValue(5);
            var luma_adj_name = luma_adj.name = "luma key adjustments";  
            var luma_blu = lwLayer.property("Effects").addProperty("Slider Control")
            var luma_blur_name = luma_blu.name = "bg extra blurriness";
            // Add expressions to effects.
            lumaLevels("Alpha Input Black").expression = "comp(\"" + myComp.name + "\").layer(\"" + lwComp.name + "\").effect(\"" + luma_adj_name + "\")" + "(\"Alpha Input Black\")";
            lumaLevels("Alpha Input White").expression = "comp(\"" + myComp.name + "\").layer(\"" + lwComp.name + "\").effect(\"" + luma_adj_name + "\")" + "(\"Alpha Input White\")";
            lumaLevels("Alpha Gamma").expression = "comp(\"" + myComp.name + "\").layer(\"" + lwComp.name + "\").effect(\"" + luma_adj_name + "\")" + "(\"Alpha Gamma\")";
            var lumaBlur = lumakeyLayer.property("Effects").addProperty("Fast Blur");
            lumaBlur("Blurriness").expression =  "comp(\"" + myComp.name + "\").layer(\"" + lwComp.name + "\").effect(\"" + luma_blur_name + "\")" + "(\"Slider\")";
            } // if(UV6 == true){
        } // if(myPalette.grp.one.luma.value == true){

app.endUndoGroup();  

} // function lightwrap (palObject) {

////////////////////////////////////////////////////////////////////////////////////////////////////// Lightwrap function end //////////////////////////////////////////////////////////////////////////////////////////////////////////////